package com.link.core.basic.es.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.link.core.basic.es.annotation.EsIndexAlias;
import com.link.core.basic.es.parse.EsAggrParse;
import com.link.core.basic.es.parse.EsDistinctParse;
import com.link.core.basic.es.parse.EsFilterParse;
import com.link.core.basic.es.parse.EsOauthParse;
import com.link.core.basic.es.parse.EsSorterParse;
import com.link.core.basic.es.util.EsCacheUtils;
import com.link.core.basic.es.util.EsUtil;
import com.link.core.basic.model.BasicModel;
import com.link.core.basic.service.BasicServiceException;
import com.link.core.basic.service.BasicServiceImpl;
import com.link.core.basic.service.KeyGenerateService;
import com.link.core.cllog.LCLogger;
import com.link.core.util.ReflectUtil;
import com.link.core.util.StringUtils;
import com.link.core.util.json.ObjectOperaUtils;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.elasticsearch.action.DocWriteResponse;
import org.elasticsearch.action.DocWriteResponse.Result;
import org.elasticsearch.action.bulk.BulkItemResponse;
import org.elasticsearch.action.delete.DeleteRequest;
import org.elasticsearch.action.delete.DeleteResponse;
import org.elasticsearch.action.get.GetRequest;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.index.IndexResponse;
import org.elasticsearch.action.search.ClearScrollRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.search.SearchScrollRequest;
import org.elasticsearch.action.update.UpdateRequest;
import org.elasticsearch.action.update.UpdateResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.core.CountRequest;
import org.elasticsearch.client.core.CountResponse;
import org.elasticsearch.common.document.DocumentField;
import org.elasticsearch.core.TimeValue;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.reindex.BulkByScrollResponse;
import org.elasticsearch.index.reindex.UpdateByQueryRequest;
import org.elasticsearch.join.query.HasChildQueryBuilder;
import org.elasticsearch.join.query.HasParentQueryBuilder;
import org.elasticsearch.script.Script;
import org.elasticsearch.script.ScriptType;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.aggregations.bucket.terms.Terms;
import org.elasticsearch.search.aggregations.metrics.NumericMetricsAggregation;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.SortBuilder;
import org.elasticsearch.xcontent.XContentType;
import org.springframework.util.CollectionUtils;

public class BasicEsServiceImpl<T extends BasicModel> implements BasicEsService<T> {
   @Resource
   private EsFilterParse esFilterParse;
   @Resource
   private EsSorterParse esSorterParse;
   @Resource
   private EsOauthParse esOauthParse;
   @Resource
   private EsDistinctParse esDistinctParse;
   @Resource
   private RestHighLevelClient restHighLevelClient;
   @Resource
   private EsCacheUtils esCacheUtils;
   @Resource
   private EsAggrParse esAggrParse;
   private static final Logger logger = LogManager.getLogger(BasicEsServiceImpl.class);
   public static ThreadLocal<String> ES_INDEX = new ThreadLocal();
   private Class<T> modelClazz = this.getEntityClazz(this.getClass());
   @Resource
   protected KeyGenerateService keyGenerateService;

   public BasicEsServiceImpl() {
      if (this.modelClazz == null) {
         throw new BasicServiceException("初始化" + this.getClass() + "时，获取当前泛型T的真实类型失败，modelClazz为空");
      }
   }

   public List<T> queryByExamplePage(T entity) throws Exception {
      List var4;
      try {
         this.beforeQueryExample(entity);
         this.beforeQueryAllOrExample(entity);
         this.checkEsIndex(entity);
         SearchSourceBuilder sourceBuilder = this.buildQueryCondition(entity);
         this.beforeExecuteQuery(sourceBuilder, entity);
         List<T> list = this.executeQuery(sourceBuilder, entity);
         if (!entity.getOnlyCountFlag()) {
            this.handleQueryResult(list, entity);
            this.afterExecuteQuery(list);
            this.afterExecuteQueryOrScroll(list);
         }

         var4 = list;
      } finally {
         ES_INDEX.remove();
      }

      return var4;
   }

   public List<T> queryAll(T entity) throws Exception {
      List var3;
      try {
         this.beforeQueryAll(entity);
         this.beforeQueryAllOrExample(entity);
         List<T> list = this.scrollQueryByExamplePage(entity);
         if (!CollectionUtils.isEmpty(list) && !StringUtils.isBlank(((BasicModel)list.get(0)).getEsScrollId())) {
            String scrollId = ((BasicModel)list.get(0)).getEsScrollId();
            entity.setEsScrollId(scrollId);

            while(true) {
               List<T> resultList = this.scrollQueryByExamplePage(entity);
               if (CollectionUtils.isEmpty(resultList) || StringUtils.isBlank(((BasicModel)list.get(0)).getEsScrollId())) {
                  resultList = list;
                  return resultList;
               }

               list.addAll(resultList);
            }
         }

         var3 = list;
      } finally {
         ES_INDEX.remove();
      }

      return var3;
   }

   public List<T> queryAllByAggregation(T entity) throws Exception {
      List var4;
      try {
         this.beforeAggrAll(entity);
         this.checkEsIndex(entity);
         SearchSourceBuilder sourceBuilder = this.buildQueryCondition(entity);
         this.esAggrParse.parseFields(sourceBuilder, entity);
         this.beforeExecuteQuery(sourceBuilder, entity);
         List<T> list = this.executeAggrQuery(sourceBuilder, entity);
         this.afterExecuteQueryByAggregation(list);
         var4 = list;
      } finally {
         ES_INDEX.remove();
      }

      return var4;
   }

   public void insert(T entity) throws Exception {
      this.beforInsert(entity);
      this.beforUpsert(entity);
      this.postDefaultValue(entity);
      this.checkEsIndex(entity);
      if (StringUtils.isBlank(entity.getId())) {
         entity.setId(this.keyGenerateService.keyGenerate());
      } else {
         boolean idExists = this.checkIdExists(entity.getEsIndex(), entity.getId());
         if (idExists) {
            throw new BasicServiceException("当前数据已经存在，不允许新增");
         }
      }

      if (!this.checkData(entity)) {
         throw new BasicServiceException("[" + this.modelClazz.getSimpleName() + "]校验不通过，请检查数据！");
      } else {
         IndexRequest indexRequest = new IndexRequest();
         ((IndexRequest)indexRequest.index(entity.getEsIndex())).id(String.valueOf(entity.getId())).source(JSONObject.toJSONString(entity), XContentType.JSON);

         try {
            IndexResponse response = this.restHighLevelClient.index(indexRequest, RequestOptions.DEFAULT);
            this.hadnlerResult(response);
         } catch (Exception var4) {
            Exception e = var4;
            this.hadnlerResult(entity, e.getMessage(), "insert");
         }

         this.afterInsert(entity);
         this.afterUpsert(entity);
      }
   }

   public void batchInsert(List<T> list) throws Exception {
      for(int i = 0; list != null && i < list.size(); ++i) {
         this.insert((BasicModel)list.get(i));
      }

   }

   public void batchUpsert(List<T> list) throws Exception {
      for(int i = 0; list != null && i < list.size(); ++i) {
         this.upsert((BasicModel)list.get(i));
      }

   }

   public T preDefaultValue(T entity) throws Exception {
      T newEntity = (BasicModel)this.modelClazz.newInstance();
      newEntity.setId(this.keyGenerateService.keyGenerate());
      return newEntity;
   }

   public T postDefaultValue(T entity) throws Exception {
      return entity;
   }

   public void upsert(T entity) throws Exception {
      if (this.isInsertFlag(entity)) {
         this.insert(entity);
      } else {
         this.update(entity);
      }

   }

   public void update(T entity) throws Exception {
      this.beforUpdate(entity);
      this.beforUpsert(entity);
      if (!this.checkData(entity)) {
         throw new BasicServiceException("[" + this.modelClazz.getSimpleName() + "]校验不通过，请检查数据！");
      } else {
         this.checkId(entity);
         this.checkEsIndex(entity);
         String updateFields = entity.getUpdateFields();
         String updatedContent = StringUtils.isBlank(updateFields) ? JSONObject.toJSONString(entity) : ObjectOperaUtils.getIncludeFieldString(entity, updateFields);
         UpdateRequest updateRequest = (new UpdateRequest(entity.getEsIndex(), entity.getId())).doc(updatedContent, XContentType.JSON);

         try {
            UpdateResponse response = this.restHighLevelClient.update(updateRequest, RequestOptions.DEFAULT);
            this.hadnlerResult(response);
         } catch (Exception var6) {
            Exception e = var6;
            this.hadnlerResult(entity, e.getMessage(), "update");
         }

         this.afterUpdate(entity);
         this.afterUpsert(entity);
      }
   }

   public void updateByCondition(T entity) throws Exception {
      this.checkEsIndex(entity);
      SearchSourceBuilder sourceBuilder = this.buildQueryCondition(entity);
      this.beforUpdateByCondition(sourceBuilder, entity);
      UpdateByQueryRequest updateByQueryRequest = new UpdateByQueryRequest(new String[]{entity.getEsIndex()});
      updateByQueryRequest.setQuery(sourceBuilder.query());
      String updateFields = entity.getUpdateFields();
      if (StringUtils.isBlank(updateFields)) {
         throw new BasicServiceException("更新字段信息为空");
      } else {
         List<String> fieldNames = Arrays.asList(updateFields.split(","));
         Map<String, Object> updateColumns = new HashMap();
         List<Field> allFields = ReflectUtil.getAllFields(entity.getClass());
         Map<String, List<Field>> collect = (Map)allFields.stream().filter((s) -> {
            return fieldNames.contains(s.getName());
         }).collect(Collectors.groupingBy(Field::getName));
         Iterator var9 = collect.keySet().iterator();

         while(var9.hasNext()) {
            String fieldName = (String)var9.next();
            Field field = (Field)((List)collect.get(fieldName)).get(0);
            field.setAccessible(true);
            updateColumns.put(fieldName, field.get(entity));
         }

         StringBuilder scriptBuild = new StringBuilder();
         Iterator var17 = updateColumns.entrySet().iterator();

         while(var17.hasNext()) {
            Map.Entry<String, Object> entry = (Map.Entry)var17.next();
            String key = (String)entry.getKey();
            scriptBuild.append("ctx._source.").append(key).append("=").append("params.").append(key).append(";");
         }

         Script script = new Script(ScriptType.INLINE, "painless", scriptBuild.toString(), updateColumns);
         if (StringUtils.isNotBlank(script.toString())) {
            updateByQueryRequest.setScript(script);
            logger.info("条件更新：索引 {}，script {}", entity.getEsIndex(), updateByQueryRequest.getScript());
            int retryNum = 3;

            while(retryNum > 0) {
               try {
                  BulkByScrollResponse response = this.restHighLevelClient.updateByQuery(updateByQueryRequest, RequestOptions.DEFAULT);
                  if (response.getUpdated() > 0L) {
                     retryNum = 0;
                     logger.info("更新了" + response.getUpdated() + "条数据");
                  } else if (CollectionUtils.isEmpty(response.getBulkFailures())) {
                     retryNum = 0;
                     logger.warn("本次更新没有被影响的记录");
                  } else {
                     Iterator var13 = response.getBulkFailures().iterator();

                     while(var13.hasNext()) {
                        BulkItemResponse.Failure bulkFailure = (BulkItemResponse.Failure)var13.next();
                        logger.error("更新错误信息" + bulkFailure.getMessage());
                     }

                     --retryNum;
                  }
               } catch (Exception var15) {
                  LCLogger.withException(var15);
                  this.hadnlerResult(entity, var15.getMessage(), "update");
               }
            }
         } else {
            logger.debug("es 更新脚本为空！");
         }

         this.afterUpdateByCondition(entity);
      }
   }

   public void reloadById(T entity) throws Exception {
      this.beforReloadById(entity);
      if (StringUtils.isBlank(entity.getId())) {
         entity.setId(this.keyGenerateService.keyGenerate());
      }

      this.checkEsIndex(entity);
      List<Map<String, Object>> data = EsUtil.queryDataById(entity, entity.getEsIndex());
      EsUtil.querySubNestDataById(data, entity.getEsIndex());
      Map<String, Object> esConfigMap = EsCacheUtils.getEsConfigCache(entity.getEsIndex());
      Iterator var4 = data.iterator();

      while(var4.hasNext()) {
         Map<String, Object> item = (Map)var4.next();
         IndexRequest indexRequest = new IndexRequest();
         String parentField = "";
         Object childConfig;
         if ("parent".equals(entity.getEsDocShipFlag())) {
            label43: {
               childConfig = esConfigMap.get("parenMark");
               Object shipJoinField = esConfigMap.get("shipJoinField");
               if (!Objects.isNull(childConfig) && !StringUtils.isBlank(childConfig.toString())) {
                  if (!Objects.isNull(shipJoinField) && !StringUtils.isBlank(shipJoinField.toString())) {
                     Map<String, String> map = new HashMap();
                     map.put("name", childConfig.toString());
                     item.put(shipJoinField.toString(), map);
                     item.put(shipJoinField + "Relations", childConfig);
                     break label43;
                  }

                  throw new BasicServiceException("父es文档配置信息错误，请联系管理员");
               }

               throw new BasicServiceException("父es文档配置信息错误，请联系管理员");
            }
         } else if ("child".equals(entity.getEsDocShipFlag())) {
            childConfig = esConfigMap.get("childConfig");
            if (Objects.isNull(childConfig)) {
               throw new BasicServiceException("更新子文档信息，需要给定子文档标识");
            }

            JSONArray objects = JSONArray.parseArray(JSONArray.toJSONString(childConfig));
            List<Object> childMarks = (List)objects.stream().filter((s) -> {
               return entity.getEsDocShipMark().equals(JSONObject.parseObject(JSON.toJSONString(s)).getString("childMark"));
            }).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(childMarks)) {
               throw new BasicServiceException("找不到子文档【" + entity.getEsDocShipMark() + "】的配置信息");
            }

            parentField = JSONObject.parseObject(JSON.toJSONString(childMarks.get(0))).getString("parentField");
            String shipJoinField = JSONObject.parseObject(JSON.toJSONString(childMarks.get(0))).getString("shipJoinField");
            if (StringUtils.isBlank(parentField) || StringUtils.isBlank(shipJoinField)) {
               throw new BasicServiceException("配置【" + entity.getEsIndex() + "】索引的子文档【" + entity.getEsDocShipMark() + "】配置信息有误，请您联系管理员配置");
            }

            Map<String, String> map = new HashMap();
            map.put("name", entity.getEsDocShipMark());
            map.put("parent", EsUtil.getParentFieldValue(item, parentField));
            item.put(shipJoinField, map);
            item.put(shipJoinField + "Relations", entity.getEsDocShipMark());
         }

         ((IndexRequest)indexRequest.index(entity.getEsIndex())).id(String.valueOf(item.get("id"))).source(JSONObject.toJSONString(item), XContentType.JSON);
         if ("child".equals(entity.getEsDocShipFlag())) {
            indexRequest.routing(EsUtil.getParentFieldValue(item, parentField));
         }

         IndexResponse response = this.restHighLevelClient.index(indexRequest, RequestOptions.DEFAULT);
         this.hadnlerResult(response);
         this.afterReloadById(entity);
      }

   }

   public void batchReload(List<T> list) throws Exception {
      for(int i = 0; list != null && i < list.size(); ++i) {
         this.reloadById((BasicModel)list.get(i));
      }

   }

   public T queryById(T entity) throws Exception {
      this.checkId(entity);
      this.checkEsIndex(entity);
      GetRequest getRequest = new GetRequest(entity.getEsIndex(), entity.getId());
      GetResponse response = this.restHighLevelClient.get(getRequest, RequestOptions.DEFAULT);
      if (!response.isExists()) {
         return null;
      } else {
         String sourceAsString = response.getSourceAsString();
         JSONObject itemJson = JSON.parseObject(sourceAsString);
         if (!CollectionUtils.isEmpty(response.getFields())) {
            Iterator var6 = response.getFields().entrySet().iterator();

            while(var6.hasNext()) {
               Map.Entry<String, DocumentField> entry = (Map.Entry)var6.next();
               itemJson.put((String)entry.getKey(), ((DocumentField)entry.getValue()).getValues().get(0));
            }
         }

         T resultItem = (BasicModel)JSON.parseObject(itemJson.toJSONString(), entity.getClass());
         return resultItem;
      }
   }

   public T queryById(String entityId) throws Exception {
      T entity = (BasicModel)this.modelClazz.newInstance();
      entity.setId(entityId);
      return this.queryById(entity);
   }

   public void deleteById(T entity) throws Exception {
      this.beforDelete(entity);
      this.checkId(entity);
      this.checkEsIndex(entity);
      DeleteRequest deleteRequest = new DeleteRequest(entity.getEsIndex(), entity.getId());

      try {
         DeleteResponse response = this.restHighLevelClient.delete(deleteRequest, RequestOptions.DEFAULT);
         this.hadnlerResult(response);
      } catch (Exception var4) {
         Exception e = var4;
         this.hadnlerResult(entity, e.getMessage(), "delete");
      }

      this.afterDelete(entity);
   }

   public void deleteById(String entityId) throws Exception {
      T entity = (BasicModel)this.modelClazz.newInstance();
      entity.setId(entityId);
      this.deleteById(entity);
   }

   public void batchDelete(List<T> list) throws Exception {
      for(int i = 0; list != null && i < list.size(); ++i) {
         this.deleteById((BasicModel)list.get(i));
      }

   }

   public boolean checkData(T entity) throws Exception {
      return true;
   }

   public void beforUpdate(T entity) throws Exception {
   }

   public void afterUpdate(T entity) throws Exception {
   }

   public void beforUpdateByCondition(SearchSourceBuilder sourceBuilder, T entity) throws Exception {
   }

   public void afterUpdateByCondition(T entity) throws Exception {
   }

   public void beforInsert(T entity) throws Exception {
   }

   public void afterInsert(T entity) throws Exception {
   }

   public void beforDelete(T entity) throws Exception {
   }

   public void beforUpsert(T entity) throws Exception {
   }

   public void afterUpsert(T entity) throws Exception {
   }

   public void beforReloadById(T entity) throws Exception {
   }

   public void afterReloadById(T entity) throws Exception {
   }

   public boolean isInsertFlag(T entity) throws Exception {
      String statusFlag = entity.getRow_status();
      if (StringUtils.equals(statusFlag, "NEW")) {
         return true;
      } else if (StringUtils.equals(statusFlag, "UPDATE")) {
         return false;
      } else {
         throw new BasicServiceException("[" + this.modelClazz.getSimpleName() + "]无法识别当前操作状态。");
      }
   }

   public void batchUpdate(List<T> list) throws Exception {
   }

   public void afterDelete(T entity) throws Exception {
   }

   protected boolean checkIdExists(String indexName, String id) throws IOException {
      GetRequest getRequest = new GetRequest(indexName, id);
      return this.restHighLevelClient.exists(getRequest, RequestOptions.DEFAULT);
   }

   protected void hadnlerResult(DocWriteResponse response) {
      if (response.getResult() == Result.CREATED) {
         logger.info("索引文档创建成功");
      } else if (response.getResult() == Result.UPDATED) {
         logger.info("索引文档更新成功");
      } else if (response.getResult() == Result.NOOP) {
         logger.info("索引文档未发生变化");
      } else {
         if (response.getResult() != Result.DELETED) {
            if (response.getResult() == Result.NOT_FOUND) {
               logger.info("未找到索引文档");
               throw new BasicServiceException("未找到索引文档");
            }

            LCLogger.withException(JSONObject.toJSONString(response));
            throw new BasicServiceException("操作索引文档异常，请检查");
         }

         logger.info("索引文档删除成功");
      }

   }

   protected void hadnlerResult(T entity, String responseStr, String operaType) {
      if (responseStr.contains("response=")) {
         int responseIndex = responseStr.indexOf("response=");
         String responseSubstring = responseStr.substring(responseIndex + 9);
         String response = responseSubstring.substring(0, responseSubstring.length() - 1);
         String message;
         if (!response.contains("200 OK") && !response.contains("201 Created")) {
            message = "insert".equals(operaType) ? "创建文档失败" : ("update".equals(operaType) ? "更新文档失败" : "删除文档失败");
            throw new BasicServiceException(message);
         } else {
            message = "insert".equals(operaType) ? "创建文档" : ("update".equals(operaType) ? "更新文档" : "删除文档");
            logger.info(message + "【" + entity.getId() + "】成功");
         }
      } else {
         String message = "insert".equals(operaType) ? "创建文档失败" : ("update".equals(operaType) ? "更新文档失败" : "删除文档失败");
         throw new BasicServiceException(message);
      }
   }

   public void checkId(T entity) throws Exception {
      if (entity == null) {
         throw new BasicServiceException(this.modelClazz.getSimpleName() + "不能为空！");
      } else if (null == entity.getId()) {
         throw new BasicServiceException(this.modelClazz.getSimpleName() + "的ID不能为空！");
      }
   }

   private Class<T> getEntityClazz(Class<?> clazz) {
      if (clazz == null) {
         return null;
      } else if (Object.class == clazz) {
         return null;
      } else {
         Type genericSuperclass = clazz.getGenericSuperclass();
         if (genericSuperclass instanceof ParameterizedType) {
            ParameterizedType parameterizedType = (ParameterizedType)genericSuperclass;
            Type[] actualTypeArguments = parameterizedType.getActualTypeArguments();
            return (Class)actualTypeArguments[0];
         } else {
            boolean isFather = BasicServiceImpl.class.isAssignableFrom(clazz);
            if (isFather) {
               Class<?> superClass = clazz.getSuperclass();
               return this.getEntityClazz(superClass);
            } else {
               return null;
            }
         }
      }
   }

   public void handleQueryResult(List<T> list, T entity) {
      if (!CollectionUtils.isEmpty(list)) {
         int total = list.size();
         if (!entity.getTotalFlag() && !entity.getOnlyCountFlag()) {
            entity.setTotal(total);
         }

         if (total > entity.getRows() && entity.getPageFlag()) {
            list.remove(total - 1);
         }

      }
   }

   public List<T> scrollQueryByExamplePage(T entity) throws Exception {
      List var4;
      try {
         SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
         if (StringUtils.isBlank(entity.getEsScrollId())) {
            this.beforeScrollQuery(entity);
            this.beforeQueryAllOrExample(entity);
            entity.setEsScrollFlag(true);
            this.checkEsIndex(entity);
            sourceBuilder = this.buildQueryCondition(entity);
            this.beforeScrollQuery(sourceBuilder);
         }

         List<T> list = this.executeScrollQuery(sourceBuilder, entity);
         this.afterScrollQuery(list);
         this.afterExecuteQueryOrScroll(list);
         var4 = list;
      } finally {
         ES_INDEX.remove();
      }

      return var4;
   }

   public void beforeQueryExample(T entity) throws Exception {
   }

   public void beforeQueryAllOrExample(T entity) throws Exception {
   }

   public void beforeQueryAll(T entity) throws Exception {
   }

   public void beforeAggrAll(T entity) throws Exception {
   }

   public void beforeScrollQuery(T entity) throws Exception {
   }

   public void beforeScrollQuery(SearchSourceBuilder sourceBuilder) {
   }

   public void beforeExecuteQuery(SearchSourceBuilder sourceBuilder, T entity) {
   }

   public void afterExecuteQuery(List<T> list) {
   }

   public void afterScrollQuery(List<T> list) {
   }

   public void afterExecuteQueryOrScroll(List<T> list) {
   }

   private void afterExecuteQueryByAggregation(List<T> list) {
   }

   protected void checkEsIndex(T entity) throws Exception {
      EsIndexAlias esAnnotation = (EsIndexAlias)entity.getClass().getAnnotation(EsIndexAlias.class);
      if (null != esAnnotation && !StringUtils.isBlank(esAnnotation.value())) {
         String esIndexAlias = esAnnotation.value();
         new JSONObject();
         JSONObject jsonObject;
         if (esIndexAlias.contains("||")) {
            String[] list = esIndexAlias.split("\\|\\|");
            jsonObject = this.esCacheUtils.getConfigByIndexAlias(list[0]);
            if (jsonObject.isEmpty() || StringUtils.isBlank(jsonObject.getString("target_table_name"))) {
               throw new BasicServiceException("当前数据转化配置中不存在有效的配置信息，索引别名：" + entity.getEsIndexAlias());
            }

            List<String> esIndexList = new ArrayList();
            String[] var7 = list;
            int var8 = list.length;

            for(int var9 = 0; var9 < var8; ++var9) {
               String esIndexAlia = var7[var9];
               esIndexList.add(this.esCacheUtils.getConfigByIndexAlias(esIndexAlia).getString("target_table_name"));
            }

            entity.setEsIndex(jsonObject.getString("target_table_name"));
            entity.setEsIndexList(esIndexList);
         } else {
            jsonObject = this.esCacheUtils.getConfigByIndexAlias(esIndexAlias);
            if (jsonObject.isEmpty() || StringUtils.isBlank(jsonObject.getString("target_table_name"))) {
               throw new BasicServiceException("当前数据转化配置中不存在有效的配置信息，索引别名：" + entity.getEsIndexAlias());
            }

            entity.setEsIndex(jsonObject.getString("target_table_name"));
         }

         if (entity.getEsScrollFlag() && StringUtils.isBlank(entity.getEsScrollId()) && entity.getPage() != 1) {
            throw new BasicServiceException("ES滚动查询参数异常或者滚动缓存时间过期");
         } else {
            ES_INDEX.set(entity.getEsIndex());
         }
      } else {
         throw new BasicServiceException("实体类上的ES索引别名注解为空");
      }
   }

   public SearchSourceBuilder buildQueryCondition(T entity) throws Exception {
      SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
      List<QueryBuilder> queryBuilders = new ArrayList();
      QueryBuilder filterBuilder = this.esFilterParse.buildFilter(entity);
      if (null != filterBuilder) {
         queryBuilders.add(filterBuilder);
      }

      List<HasChildQueryBuilder> childFilterBuilders = this.esFilterParse.buildChildFilter(entity);
      if (!CollectionUtils.isEmpty(childFilterBuilders)) {
         queryBuilders.addAll(childFilterBuilders);
      }

      List<HasParentQueryBuilder> parentFilterBuilders = this.esFilterParse.buildParentFilter(entity);
      if (!CollectionUtils.isEmpty(parentFilterBuilders)) {
         queryBuilders.addAll(parentFilterBuilders);
      }

      List<SortBuilder<?>> sortBuilderList = this.esSorterParse.buildSorter(entity);
      if (!CollectionUtils.isEmpty(sortBuilderList)) {
         sourceBuilder.sort(sortBuilderList);
      }

      QueryBuilder oauthBuilder = this.esOauthParse.buildOauth(entity);
      if (null != oauthBuilder) {
         queryBuilders.add(oauthBuilder);
      }

      if (entity.getEsScrollFlag()) {
         sourceBuilder.size(entity.getRows());
      } else {
         this.buildPage(sourceBuilder, entity.getStartRow(), entity.getRows());
      }

      this.buildQueryFields(sourceBuilder, entity);
      if (!StringUtils.isBlank(entity.getDistinctFields())) {
         this.esDistinctParse.parseFields(sourceBuilder, entity);
      }

      BoolQueryBuilder finalQueryBuilder = QueryBuilders.boolQuery();
      finalQueryBuilder.must().addAll(queryBuilders);
      sourceBuilder.query(finalQueryBuilder);
      return sourceBuilder;
   }

   public List<T> executeQuery(SearchSourceBuilder sourceBuilder, T entity) throws Exception {
      List<T> resultList = new ArrayList();
      Exception e;
      String message;
      if (entity.getTotalFlag() || entity.getOnlyCountFlag()) {
         try {
            CountRequest countRequest = new CountRequest();
            if (!CollectionUtils.isEmpty(entity.getEsIndexList())) {
               countRequest.indices((String[])entity.getEsIndexList().toArray(new String[0]));
            } else {
               countRequest.indices(new String[]{entity.getEsIndex()});
            }

            logger.info("es查询总数请求的最终条件：" + sourceBuilder);
            countRequest.query(sourceBuilder.query());
            CountResponse countResponse = this.restHighLevelClient.count(countRequest, RequestOptions.DEFAULT);
            logger.debug("总数查询结果：" + countResponse);
            entity.setTotal((int)countResponse.getCount());
            if (entity.getOnlyCountFlag()) {
               resultList.add(entity);
               return resultList;
            }
         } catch (Exception var8) {
            e = var8;
            message = e.getMessage();
            throw new BasicServiceException("查询ES数据总量异常，" + message, e);
         }
      }

      try {
         SearchRequest searchRequest = new SearchRequest();
         logger.info("es查询请求的最终条件：" + sourceBuilder);
         searchRequest.source(sourceBuilder);
         if (!CollectionUtils.isEmpty(entity.getEsIndexList())) {
            searchRequest.indices((String[])entity.getEsIndexList().toArray(new String[0]));
         } else {
            searchRequest.indices(new String[]{entity.getEsIndex()});
         }

         SearchResponse searchResponse = this.restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
         SearchHits hits = searchResponse.getHits();
         hits.forEach((item) -> {
            JSONObject itemJson = JSON.parseObject(item.getSourceAsString());
            if (!CollectionUtils.isEmpty(item.getDocumentFields())) {
               Iterator var4 = item.getDocumentFields().entrySet().iterator();

               while(var4.hasNext()) {
                  Map.Entry<String, DocumentField> entry = (Map.Entry)var4.next();
                  itemJson.put((String)entry.getKey(), ((DocumentField)entry.getValue()).getValues().get(0));
               }
            }

            T resultItem = (BasicModel)JSON.parseObject(itemJson.toJSONString(), entity.getClass());
            resultList.add(resultItem);
            logger.debug(itemJson);
         });
         return resultList;
      } catch (Exception var7) {
         e = var7;
         message = e.getMessage();
         LCLogger.withException(e);
         throw new BasicServiceException("查询ES数据异常，" + message, e);
      }
   }

   public List<T> executeScrollQuery(SearchSourceBuilder sourceBuilder, T entity) throws Exception {
      List<T> resultList = new ArrayList();
      String scrollId = entity.getEsScrollId();

      String finalScrollId;
      try {
         SearchResponse searchResponse;
         SearchHits hits;
         if (StringUtils.isBlank(scrollId)) {
            SearchRequest searchRequest = new SearchRequest();
            if (!CollectionUtils.isEmpty(entity.getEsIndexList())) {
               searchRequest.indices((String[])entity.getEsIndexList().toArray(new String[0]));
            } else {
               searchRequest.indices(new String[]{entity.getEsIndex()});
            }

            logger.info("es滚动查询请求的最终条件：" + sourceBuilder);
            searchRequest.source(sourceBuilder);
            searchRequest.scroll(TimeValue.timeValueMinutes(60L));
            searchResponse = this.restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
            scrollId = searchResponse.getScrollId();
            hits = searchResponse.getHits();
            logger.info("滚动查询返回信息" + JSONObject.toJSONString(hits));
            logger.info("滚动查询总数" + (Objects.isNull(hits.getTotalHits()) ? 0L : hits.getTotalHits().value));
            entity.setCurrentCount(Objects.isNull(hits.getTotalHits()) ? 0 : Integer.parseInt(hits.getTotalHits().value + ""));
            logger.debug("es滚动查询请求返回的scrollId：{}, 返回的总条数：{}", scrollId, hits.getHits().length);
         } else {
            SearchScrollRequest scrollRequest = new SearchScrollRequest(scrollId);
            scrollRequest.scroll(TimeValue.timeValueMinutes(60L));
            searchResponse = this.restHighLevelClient.scroll(scrollRequest, RequestOptions.DEFAULT);
            hits = searchResponse.getHits();
            logger.debug("es滚动查询请求返回的scrollId：{}, 返回的总条数：{}", scrollId, hits.getHits().length);
         }

         if (hits.getHits().length > 0) {
            finalScrollId = scrollId;
            hits.forEach((item) -> {
               JSONObject itemJson = JSON.parseObject(item.getSourceAsString());
               if (!CollectionUtils.isEmpty(item.getDocumentFields())) {
                  Iterator var5 = item.getDocumentFields().entrySet().iterator();

                  while(var5.hasNext()) {
                     Map.Entry<String, DocumentField> entry = (Map.Entry)var5.next();
                     itemJson.put((String)entry.getKey(), ((DocumentField)entry.getValue()).getValues().get(0));
                  }
               }

               T resultItem = (BasicModel)JSON.parseObject(itemJson.toJSONString(), entity.getClass());
               resultItem.setEsScrollId(finalScrollId);
               resultList.add(resultItem);
            });
         } else {
            ClearScrollRequest clearScrollRequest = new ClearScrollRequest();
            clearScrollRequest.addScrollId(scrollId);

            try {
               this.restHighLevelClient.clearScroll(clearScrollRequest, RequestOptions.DEFAULT);
            } catch (IOException var8) {
               IOException e = var8;
               logger.error("滚动查询清除失败: {}", e.getMessage(), e);
            }
         }

         return resultList;
      } catch (Exception var9) {
         Exception e = var9;
         finalScrollId = e.getMessage();
         throw new BasicServiceException("滚动查询ES数据异常，" + finalScrollId, e);
      }
   }

   public void buildPage(SearchSourceBuilder sourceBuilder, Integer startRow, Integer queryRow) {
      startRow = startRow - 1;
      if (null == queryRow || queryRow.longValue() < 0L) {
         queryRow = 100;
      }

      if (queryRow.longValue() > 10000L) {
         queryRow = 9999;
      }

      queryRow = queryRow + 1;
      sourceBuilder.from(startRow);
      sourceBuilder.size(queryRow);
   }

   public void buildQueryFields(SearchSourceBuilder sourceBuilder, BasicModel basicModel) {
      if (!StringUtils.isBlank(basicModel.getQueryFields())) {
         String[] queryFieldsArray = basicModel.getQueryFields().split(",");
         sourceBuilder.fetchSource(queryFieldsArray, (String[])null);
      }
   }

   private List<T> executeAggrQuery(SearchSourceBuilder sourceBuilder, T entity) {
      List<T> resultList = new ArrayList();

      try {
         SearchRequest searchRequest = new SearchRequest();
         logger.debug("es聚合查询请求的最终条件：" + sourceBuilder);
         searchRequest.source(sourceBuilder);
         searchRequest.indices(new String[]{entity.getEsIndex()});
         SearchResponse searchResponse = this.restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
         logger.debug("es聚合查询请求的最终结果：" + searchResponse);
         if (Objects.isNull(searchResponse.getAggregations())) {
            return null;
         } else {
            Terms resultTerms = (Terms)searchResponse.getAggregations().get("result");
            if (!Objects.isNull(resultTerms) && !CollectionUtils.isEmpty(resultTerms.getBuckets())) {
               String[] groupFields = entity.getGroupFields().split(",");
               String aggregationType = entity.getAggregationType();
               JSONArray jsonResult = new JSONArray();
               Iterator var10 = resultTerms.getBuckets().iterator();

               while(var10.hasNext()) {
                  Terms.Bucket bucket = (Terms.Bucket)var10.next();
                  NumericMetricsAggregation.SingleValue aggregationResult = (NumericMetricsAggregation.SingleValue)bucket.getAggregations().get(aggregationType);
                  String[] groupFieldValues = bucket.getKeyAsString().split("-");
                  JSONObject itemObject = new JSONObject();
                  if (groupFieldValues.length <= 1) {
                     itemObject.put(groupFields[0], groupFieldValues[0]);
                  } else {
                     for(int i = 0; i < groupFields.length; ++i) {
                        itemObject.put(groupFields[i], groupFieldValues[i].substring(1, groupFieldValues[i].length() - 1));
                     }
                  }

                  itemObject.put("aggrResultValue", aggregationResult.getValueAsString());
                  jsonResult.add(itemObject);
               }

               if (!CollectionUtils.isEmpty(jsonResult)) {
                  resultList = JSONArray.parseArray(jsonResult.toJSONString(), entity.getClass());
               }

               return (List)resultList;
            } else {
               return null;
            }
         }
      } catch (Exception var16) {
         Exception e = var16;
         String message = e.getMessage();
         throw new BasicServiceException("聚合查询执行异常:" + message, e);
      }
   }
}
